@@ -23,7 +23,7 @@ __email__ = "kevin.carter@rackspace.com"
 __copyright__ = "2014 All Rights Reserved"
 __license__ = "Apache2"
 __date__ = "2014-12-25"
-__version__ = "0.0.2"
+__version__ = "0.0.8"
 __status__ = "Development Status :: 4 - Beta"
 __appname__ = "yaprt"
 __description__ = 'Repository builder for python source code'
@@ -282,12 +282,22 @@ ARGUMENTS_DICT = {
                     'default': 'max',
                     'choices': ['max', 'min']
                 },
+                'pip_bulk_operation': {
+                    'commands': [
+                        '--pip-bulk-operation'
+                    ],
+                    'help': 'Attempt to build all of the requirements in one'
+                            ' operation. Generally this is faster and will'
+                            ' create less duplicates.',
+                    'action': 'store_true',
+                    'default': False
+                },
                 'pip_index': {
                     'commands': [
                         '--pip-index'
                     ],
                     'help': 'Index URL to override the main pip index.',
-                    'default': None
+                    'default': 'https://pypi.python.org/simple/'
                 },
                 'pip_extra_index': {
                     'commands': [
@@ -377,6 +387,18 @@ ARGUMENTS_DICT = {
                             ' release.',
                     'action': 'store_true',
                     'default': False
+                },
+                'disable_version_sanity': {
+                    'commands': [
+                        '--disable-version-sanity'
+                    ],
+                    'help': 'Disabling version sanity removes version sanity'
+                            ' checking and all packages will be built "as-is".'
+                            ' This will produce a lot of duplicate packages'
+                            ' though is useful when attempting to build a'
+                            ' larger base repository.',
+                    'action': 'store_true',
+                    'default': False
                 }
             }
         },
@@ -398,6 +420,15 @@ ARGUMENTS_DICT = {
                     'help': 'Path to the repository directory.',
                     'required': True,
                     'default': None
+                },
+                'dir_exclude': {
+                    'commands': [
+                        '--dir-exclude'
+                    ],
+                    'help': 'Path to the directories that you want to exclude'
+                            ' indexes within.',
+                    'nargs': '+',
+                    'default': list()
                 }
             }
         }
