@@ -45,6 +45,7 @@ def build_wheels(args):
     wb.get_branches(report=report)
     wb.get_releases(report=report)
 
+    packages = list()
     if args['build_packages']:
         LOG.info('Building select packages: %d', len(args['build_packages']))
         # Build a given set of wheels as hard requirements.
@@ -58,23 +59,53 @@ def build_wheels(args):
 
     if args['build_requirements']:
         LOG.info('Found requirements: %d', len(wb.requirements))
-        wb.build_wheels(
-            packages=wb.requirements,
-            clean_first=args['force_clean']
-        )
+
+        # Make sure that the pip package is built first, if its included.
+        for item in wb.requirements:
+            if item.startswith('pip'):
+                wb.requirements.insert(
+                    0, wb.requirements.pop(
+                        wb.requirements.index(
+                            item
+                        )
+                    )
+                )
+            # TODO(cloudnull) Remove this when httpretty sucks less.
+            elif item.startswith('httpretty'):
+                LOG.warn(
+                    'httpretty is an awful package and is generally'
+                    ' un-buildable. Please use something else if possible.'
+                    ' The package install for httpretty is being changed to'
+                    ' "httpretty>=0.8.3" as its really the only functional'
+                    ' version of this package that has been released.'
+                )
+                wb.requirements.pop(
+                    wb.requirements.index(
+                        item
+                    )
+                )
+                wb.requirements.append('httpretty>=0.8.3')
+        packages.extend(wb.requirements)
+
+    wb.build_wheels(
+        packages=packages,
+        clean_first=args['force_clean']
+    )
 
     if args['build_branches']:
         LOG.info('Found branch packages: %d', len(wb.branches))
         wb.build_wheels(
             packages=wb.branches,
-            clean_first=args['force_clean']
+            clean_first=args['force_clean'],
+            force_iterate=True
         )
 
     if args['build_releases']:
         LOG.info('Found releases: %d', len(wb.releases))
         wb.build_wheels(
             packages=wb.releases,
-            clean_first=args['force_clean']
+            clean_first=args['force_clean'],
+            force_iterate=True
         )
 
 
@@ -158,7 +189,8 @@ class WheelBuilder(object):
         """
         utils.copy_file(src=src_file, dst=dst_file)
 
-    def _build_wheels(self, package, no_links=False, retry=False):
+    def _build_wheels(self, package=None, packages_file=None, no_links=False,
+                      retry=False):
         """Create a python wheel.
 
         The base command will timeout in 120 seconds and will create the wheels
@@ -174,6 +206,9 @@ class WheelBuilder(object):
 
         :param package: Name of a particular package to build.
         :type package: ``str``
+        :param packages_file: $PATH to the file which contains a list of
+                              packages to build.
+        :type packages_file: ``str``
         :param no_links: Enable / Disable add on links when building the wheel.
         :type no_links: ``bol``
         :param retry: Enable retry mode.
@@ -190,21 +225,23 @@ class WheelBuilder(object):
         ]
 
         if not no_links:
-            # Source the output directory by default for prebuilt wheels.
-            command.extend(['--find-links', self.args['build_output']])
-
-            if self.args['link_dir']:
-                command.extend(['--find-links', self.args['link_dir']])
-
             if self.args['pip_extra_link_dirs']:
                 for link in self.args['pip_extra_link_dirs']:
                     command.extend(['--find-links', link])
 
         if self.args['pip_no_deps']:
-            command.extend(['--no-deps'])
+            command.append('--no-deps')
+        else:
+            if self.args['pip_index']:
+                command.extend(['--index-url', self.args['pip_index']])
+
+            if self.args['pip_extra_index']:
+                command.extend(
+                    ['--extra-index-url', self.args['pip_extra_index']]
+                )
 
         if self.args['pip_no_index']:
-            command.extend(['--no-index'])
+            command.append('--no-index')
 
         if self.args['build_dir']:
             build_dir = self.args['build_dir']
@@ -213,16 +250,13 @@ class WheelBuilder(object):
             build_dir = tempfile.mkstemp(prefix='orb_')
             command.extend(['--build', build_dir])
 
-        if self.args['pip_index']:
-            command.extend(['--index-url', self.args['pip_index']])
-
-        if self.args['pip_extra_index']:
-            command.extend(['--extra-index-url', self.args['pip_extra_index']])
-
         if self.args['debug'] is True:
             command.append('--verbose')
 
-        command.append('"%s"' % utils.stip_quotes(item=package))
+        if packages_file:
+            command.extend(['--requirement', packages_file])
+        else:
+            command.append('"%s"' % utils.stip_quotes(item=package))
         try:
             output, success = self.shell_cmds.run_command(
                 command=' '.join(command)
@@ -231,17 +265,32 @@ class WheelBuilder(object):
                 raise OSError(output)
         except OSError as exp:
             if not retry:
-                LOG.error(
+                LOG.warn(
                     'Failed to process wheel build: "%s", other data: "%s"'
                     ' Trying again without defined link lookups.',
-                    package,
+                    package or packages_file,
                     str(exp)
                 )
-                self._build_wheels(package, no_links=True, retry=True)
+
+                # Remove the build directory when failed.
+                utils.remove_dirs(build_dir)
+
+                if package:
+                    self._build_wheels(
+                        package=package,
+                        no_links=True,
+                        retry=True
+                    )
+                else:
+                    self._build_wheels(
+                        packages_file=packages_file,
+                        no_links=True,
+                        retry=True
+                    )
             else:
                 LOG.exception(
                     'Failed to process wheel build: "%s", other data: "%s"',
-                    package,
+                    package or packages_file,
                     str(exp)
                 )
         else:
@@ -318,23 +367,37 @@ class WheelBuilder(object):
 
         :returns: ``list``
         """
-        _requirements = dict()
+
         if not requirements_list:
             requirements_list = self.requirements
 
-        for requirement in set(requirements_list):
+        # Set the incoming requirements list.
+        requirements_list = set(requirements_list)
+
+        # Check if version sanity checking is disabled.
+        if self.args['disable_version_sanity']:
+            LOG.warn('Version sanity checking has been disabled.')
+            # If disabled return a sorted set list of requirements.
+            return sorted(requirements_list)
+
+        # Create the base requirement dictionary.
+        _requirements = dict()
+        for requirement in requirements_list:
             name, versions = self._requirement_name(requirement)
             if name in _requirements:
                 req = _requirements[name]
             else:
                 req = _requirements[name] = list()
-
             req.extend(versions)
 
+        # Begin sorting the packages.
         packages = list()
         for pkg_name, versions in _requirements.items():
+            # Set the list of versions but convert it back to a list for use
+            # in a deque.
+            versions = list(set(versions))
             vds = dict([(i, list()) for i in VERSION_DESCRIPTORS])
-            q = collections.deque(list(set(versions)))
+            q = collections.deque(versions)
             while q:
                 _version = q.pop()
                 for version_descriptor in vds.keys():
@@ -376,7 +439,7 @@ class WheelBuilder(object):
                     else:
                         packages.append(pkg_name)
 
-        return sorted(packages)
+        return sorted(set(packages))
 
     def _pop_items(self, found_repos, list_items):
         """Remove items within a list.
@@ -386,9 +449,17 @@ class WheelBuilder(object):
         :param list_items: Name of list within the class to get.
         :type list_items: ``str``
         """
-        item_list = getattr(self, list_items)
-        for found_repo in found_repos:
-            item_list.pop(item_list.index(found_repo))
+        if self.args['disable_version_sanity']:
+            if found_repos:
+                LOG.warn(
+                    'Version sanity checking is disabled. At present the'
+                    ' following potentially duplicate and or conflicting'
+                    ' packages were not removed. Items: "%s"', found_repos
+                )
+        else:
+            item_list = getattr(self, list_items)
+            for found_repo in found_repos:
+                item_list.pop(item_list.index(found_repo))
 
     def _pop_requirements(self, release):
         """Remove requirement items that are within a requirements list.
@@ -430,7 +501,7 @@ class WheelBuilder(object):
             for repo_branch in repo['branches'].values():
                 if repo_branch.get('requirements'):
                     for key, value in repo_branch['requirements'].items():
-                        self.requirements.extend(value)
+                        self.requirements.extend([i.lower() for i in value])
         else:
             self.requirements = self.sort_requirements()
 
@@ -544,12 +615,21 @@ class WheelBuilder(object):
         :param package: Name of a particular package to build.
         :type package: ``str``
         """
+        # Set the name of the package from an expected type, git+ or string.
+        if 'git+' in package:
+            name = utils.git_pip_link_parse(repo=package)[0]
+        else:
+            name = self._requirement_name(package)[0]
+
+        name = name.replace('-', '_').lower()
+        LOG.debug('Checking for package name [ %s ] in link directory.', name)
         for file_name in utils.get_file_names(self.args['link_dir']):
-            if package == os.path.basename(file_name).split('-')[0]:
-                LOG.info('Removed link item from cleanup "%s"', package)
+            base_file_name = os.path.basename(file_name).split('-')[0].lower()
+            if name == base_file_name:
+                LOG.info('Removed link item from cleanup "%s"', file_name)
                 os.remove(file_name)
 
-    def build_wheels(self, packages, clean_first=False):
+    def build_wheels(self, packages, clean_first=False, force_iterate=False):
         """Create python wheels from a list of packages.
 
         This method will build all of the wheels from a list of packages. Once
@@ -559,13 +639,30 @@ class WheelBuilder(object):
 
         :param packages: List of packages to build.
         :type packages: ``list``
+        :param clean_first: Enable a search and clean for existing package
+        :type clean_first: ``bol``
+        :param force_iterate: Force package iteration.
+        :type force_iterate: ``bol``
         """
         try:
-            for package in packages:
-                if clean_first and self.args['link_dir']:
+            if clean_first and self.args['link_dir']:
+                for package in packages:
                     self._package_clean(package=package)
-                self._build_wheels(package=package)
+
+            if self.args['pip_bulk_operation'] and not force_iterate:
+                req_file = os.path.join(
+                    self.args['link_dir'],
+                    'build_reqs.txt'
+                )
+                LOG.info('Requirement file being written: "%s"', req_file)
+                self.shell_cmds.mkdir_p(path=os.path.dirname(req_file))
+                with open(req_file, 'wb') as f:
+                    f.writelines(['%s\n' % i for i in packages])
+
+                self._build_wheels(packages_file=req_file)
             else:
-                self._store_pool()
+                for package in packages:
+                    self._build_wheels(package=package)
+            self._store_pool()
         finally:
             utils.remove_dirs(directory=self.args['build_output'])
